﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.IO.Ports;
using System.Timers;

namespace AlarmClock
{
    class SerialCom
    {
        public SerialPort sp;
        Timer timer = new Timer(100); //10 ms timer
        String readbuffer = "";
        bool mcu_ready = true;
        //creates a new connection
        public SerialCom(String com)
        {
            sp = new SerialPort(com, 9600, Parity.None, 8, StopBits.One);
            sp.NewLine = "\r"; 
            sp.ReadBufferSize = 500;
            timer.Elapsed += new ElapsedEventHandler(timer_Elapsed);
            

        }
        //gets alarms from MCU
        public void GetAlarms(List<Alarm> la, List<WeeklyAlarm> lwa)
        {
            if (!(sp.IsOpen == true)) sp.Open();
            sp.ReadExisting(); // ensures read buffer is empty
            readbuffer = "";
            WriteLine("LA"); // tells MCU to send list of normal alarms
            mcu_ready = false;
            timer.Start(); // allow 0.1s per alarm

            while (true)
            {
                if (mcu_ready)
                {
                    //check if we are done
                    String s = readbuffer.Substring(0, 4);
                    if (s.Equals("done"))
                    {
                        WriteLine("EN");
                        waitforOK();
                        break;
                    }

                    //parse the string sent and create a new alarm based on the string
                    System.Console.WriteLine("In :" + readbuffer);
                    Alarm a = parseNormal(readbuffer);

                    if (!la.Contains(a))
                    {
                        la.Add(a);

                    }
                    //tell it to send next alarm
                    readbuffer = "";
                    WriteLine("NE"); // tells MCU to send next alarm
                    mcu_ready = false;
                    timer.Start(); // allow 0.1s per alarm

                }

            }

            // finished getting list of all normal alarms
            la.Sort();

            // get the weekly alarms
            sp.ReadExisting(); // ensures read buffer is empty
            readbuffer = "";
            WriteLine("LW"); // tells MCU to send list of Weekly alarms
            mcu_ready = false;
            timer.Start(); // allow 0.1s per alarm

            while (true)
            {
                if (mcu_ready)
                {
                    //check if we are done
                    String s = readbuffer.Substring(0, 4);
                    if (s.Equals("done")) 
                    {
                    WriteLine("EN");
                    waitforOK();
                    break;
                    }
                    //parse the string sent and create a new alarm based on the string
                    System.Console.WriteLine("In :" + readbuffer);
                    WeeklyAlarm wa = parseWeekly(readbuffer);

                    if (!lwa.Contains(wa))
                    {
                        lwa.Add(wa);

                    }
                    //tell it to send next alarm
                    readbuffer = ""; //empty buffer
                    WriteLine("NE"); // tells MCU to send next alarm
                    mcu_ready = false;
                    timer.Start(); // allow 0.1s per alarm

                }

            }

            // gotten all weekly alarms
            lwa.Sort();

           
        }

        public void WriteLine(String c)
        {
            System.Console.WriteLine("Out =" + c);
            sp.WriteLine(c);

        }
        //converts string from MCU to weekly alarm object
        public WeeklyAlarm parseWeekly(String c)
        {
            System.Console.WriteLine(c);
            String data = c;
            String remainder = "";
            String wd = c.Substring(0, 1); // wd contains the weekday
            remainder = c.Substring(1);
            

            String h = remainder.Substring(0, 2); //data contains hour
            remainder = remainder.Substring(2); // remainder contains the min
            String m = remainder.Substring(0, 2); //data contains min
            String mes = remainder.Substring(2); // remainder contains the alarm message

            return new WeeklyAlarm(wd, h, m, mes);


        }
        //converts string from MCU to normal alarm object
        public Alarm parseNormal(String c)
        {
            System.Console.WriteLine(c);
            String data = c;
            String remainder = "";
            String y = c.Substring(0, 4); // y contains the year
            remainder = c.Substring(4);
            String M = remainder.Substring(0, 2); //M contains the month
            remainder = remainder.Substring(2);
            String d = remainder.Substring(0, 2); // d contains the day
            remainder = remainder.Substring(2);//remainder contains the time

            String h = remainder.Substring(0, 2); //data contains hour
            remainder = remainder.Substring(2); // remainder contains the min
            String m = remainder.Substring(0, 2); //data contains min
            String mes = remainder.Substring(2); // remainder contains the alarm message

            return new Alarm(y, M, d, h, m, mes);
        }

        //sends list of alarms to MCU
        public void SendAlarms(List<Alarm> la, List<WeeklyAlarm> lwa)
        {
            if (!(sp.IsOpen == true)) sp.Open();
            sp.ReadExisting(); // ensures read buffer is empty

            mcu_ready = false;
            WriteLine("SA");// tells MCU we are sending it the list of alarms
            waitforOK();
            foreach (Alarm a in la){
                
              WriteLine(a.DateInfo());
                waitforOK();
                WriteLine(a.MessageInfo());
                waitforOK();
                readbuffer = "";
            }

            WriteLine("EN");
            waitforOK();

            WriteLine("SW");// tells com we are sending weekly alarms
            waitforOK();
            foreach(WeeklyAlarm wa in lwa){
                WriteLine(wa.dayInfo());
                waitforOK();
                WriteLine(wa.MessageInfo());
                waitforOK();
            }
            WriteLine("EN");
            waitforOK();
            


        }

        //method to wait for CPU acknowledgement
        public void waitforOK()
        {
            readbuffer = "";
            while (true)
            {
                readbuffer = readbuffer + sp.ReadExisting();
                if (readbuffer.Equals("done"))
                {
                    System.Console.WriteLine("In :" + readbuffer);
                    break;
                }
            }
            
        }

        //timer to know when CPU has finished sending
        public void timer_Elapsed(object source, ElapsedEventArgs e)
        {

            String s = sp.ReadExisting();
            readbuffer = readbuffer + s;
            if (s == null || s == "")
            {
                mcu_ready = true;
                System.Console.WriteLine("In :" + readbuffer);
                timer.Stop();
            }

        }
        public void WriteData(string msg)
        {

            if (!(sp.IsOpen == true)) sp.Open();
            //send the message to the port
            sp.Write(msg);
            Console.WriteLine(msg);
        }

        public String ReadData()
        {
            String s = sp.ReadExisting();
            return s;
        }
        
        
        
            
    }
}
